@@ -1,4 +1,4 @@
-/* $Id: log-vbox.cpp 70883 2011-03-31 14:33:23Z klaus $ */
+/* $Id: log-vbox.cpp 71921 2011-05-26 08:54:27Z leonid $ */
 /** @file
  * VirtualBox Runtime - Logging configuration.
  */
@@ -461,6 +461,11 @@ RTDECL(PRTLOGGER) RTLogDefaultInit(void)
         RTLogFlags(pLogger, "enabled unbuffered");
         pLogger->fDestFlags |= RTLOGDEST_DEBUGGER;
 # endif
+# if defined(DEBUG_leo) /* Guest ring-0 as well */
+        RTLogGroupSettings(pLogger, "+drv_mouse.e.l.f+drv_miniport.e.l.f+drv_display.e.l.f");
+        RTLogFlags(pLogger, "enabled unbuffered");
+        pLogger->fDestFlags |= RTLOGDEST_DEBUGGER;
+# endif
 # if 0 /* vboxdrv logging - ATTENTION: this is what we're referring to guys! Change to '# if 1'. */
         RTLogGroupSettings(pLogger, "all=~0 -default.l6.l5.l4.l3");
         RTLogFlags(pLogger, "enabled unbuffered tid");
