@@ -1,4 +1,4 @@
-/* $Id: VBoxNetFltInternal.h 71552 2011-05-04 13:11:17Z ramshankar $ */
+/* $Id: VBoxNetFltInternal.h 71550 2011-05-04 12:54:03Z ramshankar $ */
 /** @file
  * VBoxNetFlt - Network Filter Driver (Host), Internal Header.
  */
@@ -242,7 +242,7 @@ typedef struct VBOXNETFLTINS
             /** @name Windows instance data.
              * @{ */
             /** Filter driver device context. */
-            ADAPT IfAdaptor;
+            VBOXNETFLTWIN WinIf;
 
             volatile uint32_t cModeNetFltRefs;
             volatile uint32_t cModePassThruRefs;
@@ -252,8 +252,8 @@ typedef struct VBOXNETFLTINS
 #endif
             /** The MAC address of the interface. Caching MAC for performance reasons. */
             RTMAC MacAddr;
-            /** mutex used to synchronize ADAPT init/deinit */
-            RTSEMMUTEX hAdaptMutex;
+            /** mutex used to synchronize WinIf init/deinit */
+            RTSEMMUTEX hWinIfMutex;
             /** @}  */
 # else
 #  error "PORTME"
