@@ -1,10 +1,10 @@
-/* $Id: strformat.cpp 47644 2009-05-23 15:42:57Z bird $ */
+/* $Id: strformat.cpp 67140 2010-10-28 09:27:05Z fmehnert $ */
 /** @file
  * IPRT - String Formatter.
  */
 
 /*
- * Copyright (C) 2006-2007 Sun Microsystems, Inc.
+ * Copyright (C) 2006-2007 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -22,10 +22,6 @@
  *
  * You may elect to license modified versions of this file under the
  * terms and conditions of either the GPL or the CDDL or both.
- *
- * Please contact Sun Microsystems, Inc., 4150 Network Circle, Santa
- * Clara, CA 95054 USA or visit http://www.sun.com if you need
- * additional information or have any questions.
  */
 
 
@@ -42,6 +38,8 @@
 *******************************************************************************/
 #define LOG_GROUP RTLOGGROUP_STRING
 #include <iprt/string.h>
+#include "internal/iprt.h"
+
 #include <iprt/assert.h>
 #ifdef IN_RING3
 # include <iprt/alloc.h>
@@ -153,6 +151,7 @@ RTDECL(int) RTStrFormatNumber(char *psz, uint64_t u64Value, unsigned int uiBase,
 {
     return rtStrFormatNumber(psz, *(KSIZE64 *)(void *)&u64Value, uiBase, cchWidth, cchPrecision, fFlags);
 }
+RT_EXPORT_SYMBOL(RTStrFormatNumber);
 
 
 
@@ -184,13 +183,13 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
         pachDigits = "0123456789ABCDEF";
     if (fFlags & RTSTR_F_LEFT)
         fFlags &= ~RTSTR_F_ZEROPAD;
-    if (    (fFlags & RTSTR_F_THOUSAND_SEP) 
-        &&  (   uiBase != 10 
+    if (    (fFlags & RTSTR_F_THOUSAND_SEP)
+        &&  (   uiBase != 10
              || (fFlags & RTSTR_F_ZEROPAD))) /** @todo implement RTSTR_F_ZEROPAD + RTSTR_F_THOUSAND_SEP. */
         fFlags &= ~RTSTR_F_THOUSAND_SEP;
 
     /*
-     * Determin value length
+     * Determine value length
      */
     cchValue = 0;
     if (ullValue.ulHi || (fFlags & RTSTR_F_64BIT))
@@ -284,7 +283,7 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
     {
         uint64_t    u64 = *(uint64_t *)(void *)&ullValue;
         if (fFlags & RTSTR_F_THOUSAND_SEP)
-        {    
+        {
             do
             {
                 if ((-i - 1) % 4 == 3)
@@ -294,7 +293,7 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
             } while (u64);
         }
         else
-        {    
+        {
             do
             {
                 psz[i--] = pachDigits[u64 % uiBase];
@@ -352,7 +351,8 @@ static int rtStrFormatNumber(char *psz, KSIZE64 ullValue, unsigned int uiBase, s
  * @param   pszFormat   Format string.
  * @param   InArgs      Argument list.
  */
-RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRFORMAT pfnFormat, void *pvArgFormat, const char *pszFormat, va_list InArgs)
+RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRFORMAT pfnFormat, void *pvArgFormat,
+                            const char *pszFormat, va_list InArgs)
 {
     va_list     args;
     KSIZE       cch = 0;
@@ -484,7 +484,7 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
 
 #ifndef IN_RING3
                     case 'S':   /* Unicode string as current code page -> Unicode as UTF-8 in GC/R0. */
-                        chArgSize = 'l';
+                        chArgSize = 'l'; /** @todo this is nonsensical, isn't it? */
                         /* fall thru */
 #endif
                     case 's':   /* Unicode string as utf8 */
@@ -821,6 +821,7 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                         va_copy(ArgsNested, *pArgsNested);
                         Assert(pszFormatNested);
                         cch += RTStrFormatV(pfnOutput, pvArgOutput, pfnFormat, pvArgFormat, pszFormatNested, ArgsNested);
+                        va_end(ArgsNested);
                         break;
                     }
 
@@ -832,28 +833,16 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                         if (*pszFormat != '[')
                         {
                             pszFormat--;
-                            cch += rtstrFormatRt(pfnOutput, pvArgOutput, &pszFormat, &args, cchPrecision, cchWidth, fFlags, chArgSize);
+                            cch += rtstrFormatRt(pfnOutput, pvArgOutput, &pszFormat, &args, cchWidth, cchPrecision, fFlags, chArgSize);
                         }
                         else
                         {
                             pszFormat--;
-                            cch += rtstrFormatType(pfnOutput, pvArgOutput, &pszFormat, &args, cchPrecision, cchWidth, fFlags, chArgSize);
+                            cch += rtstrFormatType(pfnOutput, pvArgOutput, &pszFormat, &args, cchWidth, cchPrecision, fFlags, chArgSize);
                         }
                         break;
                     }
 
-#ifdef RT_WITH_VBOX
-                    /*
-                     * VBox extensions.
-                     */
-                    case 'V':
-                    {
-                        pszFormat--;
-                        cch += rtstrFormatVBox(pfnOutput, pvArgOutput, &pszFormat, &args, cchPrecision, cchWidth, fFlags, chArgSize);
-                        break;
-                    }
-#endif
-
                     /*
                      * Custom format.
                      */
@@ -862,7 +851,7 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
                         if (pfnFormat)
                         {
                             pszFormat--;
-                            cch += pfnFormat(pvArgFormat, pfnOutput, pvArgOutput, &pszFormat, &args, cchPrecision, cchWidth, fFlags, chArgSize);
+                            cch += pfnFormat(pvArgFormat, pfnOutput, pvArgOutput, &pszFormat, &args, cchWidth, cchPrecision, fFlags, chArgSize);
                         }
                         break;
                     }
@@ -883,6 +872,7 @@ RTDECL(size_t) RTStrFormatV(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRF
 
     return cch;
 }
+RT_EXPORT_SYMBOL(RTStrFormatV);
 
 
 /**
@@ -909,4 +899,5 @@ RTDECL(size_t) RTStrFormat(PFNRTSTROUTPUT pfnOutput, void *pvArgOutput, PFNSTRFO
     va_end(args);
     return cch;
 }
+RT_EXPORT_SYMBOL(RTStrFormat);
 
